# Copyright 2016 Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.

from __future__ import annotations

from ..core.model import *
from .model import *


class DescribeNamespacesResult(core.Gs2Result):
    items: List[Namespace] = None
    next_page_token: str = None

    def with_items(self, items: List[Namespace]) -> DescribeNamespacesResult:
        self.items = items
        return self

    def with_next_page_token(self, next_page_token: str) -> DescribeNamespacesResult:
        self.next_page_token = next_page_token
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[DescribeNamespacesResult]:
        if data is None:
            return None
        return DescribeNamespacesResult()\
            .with_items(None if data.get('items') is None else [
                Namespace.from_dict(data.get('items')[i])
                for i in range(len(data.get('items')))
            ])\
            .with_next_page_token(data.get('nextPageToken'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "items": None if self.items is None else [
                self.items[i].to_dict() if self.items[i] else None
                for i in range(len(self.items))
            ],
            "nextPageToken": self.next_page_token,
        }


class CreateNamespaceResult(core.Gs2Result):
    item: Namespace = None

    def with_item(self, item: Namespace) -> CreateNamespaceResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[CreateNamespaceResult]:
        if data is None:
            return None
        return CreateNamespaceResult()\
            .with_item(Namespace.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class GetNamespaceStatusResult(core.Gs2Result):
    status: str = None

    def with_status(self, status: str) -> GetNamespaceStatusResult:
        self.status = status
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[GetNamespaceStatusResult]:
        if data is None:
            return None
        return GetNamespaceStatusResult()\
            .with_status(data.get('status'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "status": self.status,
        }


class GetNamespaceResult(core.Gs2Result):
    item: Namespace = None

    def with_item(self, item: Namespace) -> GetNamespaceResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[GetNamespaceResult]:
        if data is None:
            return None
        return GetNamespaceResult()\
            .with_item(Namespace.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class UpdateNamespaceResult(core.Gs2Result):
    item: Namespace = None

    def with_item(self, item: Namespace) -> UpdateNamespaceResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[UpdateNamespaceResult]:
        if data is None:
            return None
        return UpdateNamespaceResult()\
            .with_item(Namespace.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class DeleteNamespaceResult(core.Gs2Result):
    item: Namespace = None

    def with_item(self, item: Namespace) -> DeleteNamespaceResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[DeleteNamespaceResult]:
        if data is None:
            return None
        return DeleteNamespaceResult()\
            .with_item(Namespace.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class DumpUserDataByUserIdResult(core.Gs2Result):

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[DumpUserDataByUserIdResult]:
        if data is None:
            return None
        return DumpUserDataByUserIdResult()\

    def to_dict(self) -> Dict[str, Any]:
        return {
        }


class CheckDumpUserDataByUserIdResult(core.Gs2Result):
    url: str = None

    def with_url(self, url: str) -> CheckDumpUserDataByUserIdResult:
        self.url = url
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[CheckDumpUserDataByUserIdResult]:
        if data is None:
            return None
        return CheckDumpUserDataByUserIdResult()\
            .with_url(data.get('url'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "url": self.url,
        }


class CleanUserDataByUserIdResult(core.Gs2Result):

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[CleanUserDataByUserIdResult]:
        if data is None:
            return None
        return CleanUserDataByUserIdResult()\

    def to_dict(self) -> Dict[str, Any]:
        return {
        }


class CheckCleanUserDataByUserIdResult(core.Gs2Result):

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[CheckCleanUserDataByUserIdResult]:
        if data is None:
            return None
        return CheckCleanUserDataByUserIdResult()\

    def to_dict(self) -> Dict[str, Any]:
        return {
        }


class PrepareImportUserDataByUserIdResult(core.Gs2Result):
    upload_token: str = None
    upload_url: str = None

    def with_upload_token(self, upload_token: str) -> PrepareImportUserDataByUserIdResult:
        self.upload_token = upload_token
        return self

    def with_upload_url(self, upload_url: str) -> PrepareImportUserDataByUserIdResult:
        self.upload_url = upload_url
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[PrepareImportUserDataByUserIdResult]:
        if data is None:
            return None
        return PrepareImportUserDataByUserIdResult()\
            .with_upload_token(data.get('uploadToken'))\
            .with_upload_url(data.get('uploadUrl'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "uploadToken": self.upload_token,
            "uploadUrl": self.upload_url,
        }


class ImportUserDataByUserIdResult(core.Gs2Result):

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[ImportUserDataByUserIdResult]:
        if data is None:
            return None
        return ImportUserDataByUserIdResult()\

    def to_dict(self) -> Dict[str, Any]:
        return {
        }


class CheckImportUserDataByUserIdResult(core.Gs2Result):
    url: str = None

    def with_url(self, url: str) -> CheckImportUserDataByUserIdResult:
        self.url = url
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[CheckImportUserDataByUserIdResult]:
        if data is None:
            return None
        return CheckImportUserDataByUserIdResult()\
            .with_url(data.get('url'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "url": self.url,
        }


class GetProfileResult(core.Gs2Result):
    item: Profile = None

    def with_item(self, item: Profile) -> GetProfileResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[GetProfileResult]:
        if data is None:
            return None
        return GetProfileResult()\
            .with_item(Profile.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class GetProfileByUserIdResult(core.Gs2Result):
    item: Profile = None

    def with_item(self, item: Profile) -> GetProfileByUserIdResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[GetProfileByUserIdResult]:
        if data is None:
            return None
        return GetProfileByUserIdResult()\
            .with_item(Profile.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class UpdateProfileResult(core.Gs2Result):
    item: Profile = None

    def with_item(self, item: Profile) -> UpdateProfileResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[UpdateProfileResult]:
        if data is None:
            return None
        return UpdateProfileResult()\
            .with_item(Profile.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class UpdateProfileByUserIdResult(core.Gs2Result):
    item: Profile = None

    def with_item(self, item: Profile) -> UpdateProfileByUserIdResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[UpdateProfileByUserIdResult]:
        if data is None:
            return None
        return UpdateProfileByUserIdResult()\
            .with_item(Profile.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class DeleteProfileByUserIdResult(core.Gs2Result):
    item: Profile = None

    def with_item(self, item: Profile) -> DeleteProfileByUserIdResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[DeleteProfileByUserIdResult]:
        if data is None:
            return None
        return DeleteProfileByUserIdResult()\
            .with_item(Profile.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class UpdateProfileByStampSheetResult(core.Gs2Result):
    item: Profile = None

    def with_item(self, item: Profile) -> UpdateProfileByStampSheetResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[UpdateProfileByStampSheetResult]:
        if data is None:
            return None
        return UpdateProfileByStampSheetResult()\
            .with_item(Profile.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class DescribeFriendsResult(core.Gs2Result):
    items: List[FriendUser] = None
    next_page_token: str = None

    def with_items(self, items: List[FriendUser]) -> DescribeFriendsResult:
        self.items = items
        return self

    def with_next_page_token(self, next_page_token: str) -> DescribeFriendsResult:
        self.next_page_token = next_page_token
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[DescribeFriendsResult]:
        if data is None:
            return None
        return DescribeFriendsResult()\
            .with_items(None if data.get('items') is None else [
                FriendUser.from_dict(data.get('items')[i])
                for i in range(len(data.get('items')))
            ])\
            .with_next_page_token(data.get('nextPageToken'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "items": None if self.items is None else [
                self.items[i].to_dict() if self.items[i] else None
                for i in range(len(self.items))
            ],
            "nextPageToken": self.next_page_token,
        }


class DescribeFriendsByUserIdResult(core.Gs2Result):
    items: List[FriendUser] = None
    next_page_token: str = None

    def with_items(self, items: List[FriendUser]) -> DescribeFriendsByUserIdResult:
        self.items = items
        return self

    def with_next_page_token(self, next_page_token: str) -> DescribeFriendsByUserIdResult:
        self.next_page_token = next_page_token
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[DescribeFriendsByUserIdResult]:
        if data is None:
            return None
        return DescribeFriendsByUserIdResult()\
            .with_items(None if data.get('items') is None else [
                FriendUser.from_dict(data.get('items')[i])
                for i in range(len(data.get('items')))
            ])\
            .with_next_page_token(data.get('nextPageToken'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "items": None if self.items is None else [
                self.items[i].to_dict() if self.items[i] else None
                for i in range(len(self.items))
            ],
            "nextPageToken": self.next_page_token,
        }


class DescribeBlackListResult(core.Gs2Result):
    items: List[str] = None
    next_page_token: str = None

    def with_items(self, items: List[str]) -> DescribeBlackListResult:
        self.items = items
        return self

    def with_next_page_token(self, next_page_token: str) -> DescribeBlackListResult:
        self.next_page_token = next_page_token
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[DescribeBlackListResult]:
        if data is None:
            return None
        return DescribeBlackListResult()\
            .with_items(None if data.get('items') is None else [
                data.get('items')[i]
                for i in range(len(data.get('items')))
            ])\
            .with_next_page_token(data.get('nextPageToken'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "items": None if self.items is None else [
                self.items[i]
                for i in range(len(self.items))
            ],
            "nextPageToken": self.next_page_token,
        }


class DescribeBlackListByUserIdResult(core.Gs2Result):
    items: List[str] = None
    next_page_token: str = None

    def with_items(self, items: List[str]) -> DescribeBlackListByUserIdResult:
        self.items = items
        return self

    def with_next_page_token(self, next_page_token: str) -> DescribeBlackListByUserIdResult:
        self.next_page_token = next_page_token
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[DescribeBlackListByUserIdResult]:
        if data is None:
            return None
        return DescribeBlackListByUserIdResult()\
            .with_items(None if data.get('items') is None else [
                data.get('items')[i]
                for i in range(len(data.get('items')))
            ])\
            .with_next_page_token(data.get('nextPageToken'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "items": None if self.items is None else [
                self.items[i]
                for i in range(len(self.items))
            ],
            "nextPageToken": self.next_page_token,
        }


class RegisterBlackListResult(core.Gs2Result):
    item: BlackList = None

    def with_item(self, item: BlackList) -> RegisterBlackListResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[RegisterBlackListResult]:
        if data is None:
            return None
        return RegisterBlackListResult()\
            .with_item(BlackList.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class RegisterBlackListByUserIdResult(core.Gs2Result):
    item: BlackList = None

    def with_item(self, item: BlackList) -> RegisterBlackListByUserIdResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[RegisterBlackListByUserIdResult]:
        if data is None:
            return None
        return RegisterBlackListByUserIdResult()\
            .with_item(BlackList.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class UnregisterBlackListResult(core.Gs2Result):
    item: BlackList = None

    def with_item(self, item: BlackList) -> UnregisterBlackListResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[UnregisterBlackListResult]:
        if data is None:
            return None
        return UnregisterBlackListResult()\
            .with_item(BlackList.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class UnregisterBlackListByUserIdResult(core.Gs2Result):
    item: BlackList = None

    def with_item(self, item: BlackList) -> UnregisterBlackListByUserIdResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[UnregisterBlackListByUserIdResult]:
        if data is None:
            return None
        return UnregisterBlackListByUserIdResult()\
            .with_item(BlackList.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class DescribeFollowsResult(core.Gs2Result):
    items: List[FollowUser] = None
    next_page_token: str = None

    def with_items(self, items: List[FollowUser]) -> DescribeFollowsResult:
        self.items = items
        return self

    def with_next_page_token(self, next_page_token: str) -> DescribeFollowsResult:
        self.next_page_token = next_page_token
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[DescribeFollowsResult]:
        if data is None:
            return None
        return DescribeFollowsResult()\
            .with_items(None if data.get('items') is None else [
                FollowUser.from_dict(data.get('items')[i])
                for i in range(len(data.get('items')))
            ])\
            .with_next_page_token(data.get('nextPageToken'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "items": None if self.items is None else [
                self.items[i].to_dict() if self.items[i] else None
                for i in range(len(self.items))
            ],
            "nextPageToken": self.next_page_token,
        }


class DescribeFollowsByUserIdResult(core.Gs2Result):
    items: List[FollowUser] = None
    next_page_token: str = None

    def with_items(self, items: List[FollowUser]) -> DescribeFollowsByUserIdResult:
        self.items = items
        return self

    def with_next_page_token(self, next_page_token: str) -> DescribeFollowsByUserIdResult:
        self.next_page_token = next_page_token
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[DescribeFollowsByUserIdResult]:
        if data is None:
            return None
        return DescribeFollowsByUserIdResult()\
            .with_items(None if data.get('items') is None else [
                FollowUser.from_dict(data.get('items')[i])
                for i in range(len(data.get('items')))
            ])\
            .with_next_page_token(data.get('nextPageToken'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "items": None if self.items is None else [
                self.items[i].to_dict() if self.items[i] else None
                for i in range(len(self.items))
            ],
            "nextPageToken": self.next_page_token,
        }


class GetFollowResult(core.Gs2Result):
    item: FollowUser = None

    def with_item(self, item: FollowUser) -> GetFollowResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[GetFollowResult]:
        if data is None:
            return None
        return GetFollowResult()\
            .with_item(FollowUser.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class GetFollowByUserIdResult(core.Gs2Result):
    item: FollowUser = None

    def with_item(self, item: FollowUser) -> GetFollowByUserIdResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[GetFollowByUserIdResult]:
        if data is None:
            return None
        return GetFollowByUserIdResult()\
            .with_item(FollowUser.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class FollowResult(core.Gs2Result):
    item: FollowUser = None

    def with_item(self, item: FollowUser) -> FollowResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[FollowResult]:
        if data is None:
            return None
        return FollowResult()\
            .with_item(FollowUser.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class FollowByUserIdResult(core.Gs2Result):
    item: FollowUser = None

    def with_item(self, item: FollowUser) -> FollowByUserIdResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[FollowByUserIdResult]:
        if data is None:
            return None
        return FollowByUserIdResult()\
            .with_item(FollowUser.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class UnfollowResult(core.Gs2Result):
    item: FollowUser = None

    def with_item(self, item: FollowUser) -> UnfollowResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[UnfollowResult]:
        if data is None:
            return None
        return UnfollowResult()\
            .with_item(FollowUser.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class UnfollowByUserIdResult(core.Gs2Result):
    item: FollowUser = None

    def with_item(self, item: FollowUser) -> UnfollowByUserIdResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[UnfollowByUserIdResult]:
        if data is None:
            return None
        return UnfollowByUserIdResult()\
            .with_item(FollowUser.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class GetFriendResult(core.Gs2Result):
    item: FriendUser = None

    def with_item(self, item: FriendUser) -> GetFriendResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[GetFriendResult]:
        if data is None:
            return None
        return GetFriendResult()\
            .with_item(FriendUser.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class GetFriendByUserIdResult(core.Gs2Result):
    item: FriendUser = None

    def with_item(self, item: FriendUser) -> GetFriendByUserIdResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[GetFriendByUserIdResult]:
        if data is None:
            return None
        return GetFriendByUserIdResult()\
            .with_item(FriendUser.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class DeleteFriendResult(core.Gs2Result):
    item: FriendUser = None

    def with_item(self, item: FriendUser) -> DeleteFriendResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[DeleteFriendResult]:
        if data is None:
            return None
        return DeleteFriendResult()\
            .with_item(FriendUser.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class DeleteFriendByUserIdResult(core.Gs2Result):
    item: FriendUser = None

    def with_item(self, item: FriendUser) -> DeleteFriendByUserIdResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[DeleteFriendByUserIdResult]:
        if data is None:
            return None
        return DeleteFriendByUserIdResult()\
            .with_item(FriendUser.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class DescribeSendRequestsResult(core.Gs2Result):
    items: List[FriendRequest] = None
    next_page_token: str = None

    def with_items(self, items: List[FriendRequest]) -> DescribeSendRequestsResult:
        self.items = items
        return self

    def with_next_page_token(self, next_page_token: str) -> DescribeSendRequestsResult:
        self.next_page_token = next_page_token
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[DescribeSendRequestsResult]:
        if data is None:
            return None
        return DescribeSendRequestsResult()\
            .with_items(None if data.get('items') is None else [
                FriendRequest.from_dict(data.get('items')[i])
                for i in range(len(data.get('items')))
            ])\
            .with_next_page_token(data.get('nextPageToken'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "items": None if self.items is None else [
                self.items[i].to_dict() if self.items[i] else None
                for i in range(len(self.items))
            ],
            "nextPageToken": self.next_page_token,
        }


class DescribeSendRequestsByUserIdResult(core.Gs2Result):
    items: List[FriendRequest] = None
    next_page_token: str = None

    def with_items(self, items: List[FriendRequest]) -> DescribeSendRequestsByUserIdResult:
        self.items = items
        return self

    def with_next_page_token(self, next_page_token: str) -> DescribeSendRequestsByUserIdResult:
        self.next_page_token = next_page_token
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[DescribeSendRequestsByUserIdResult]:
        if data is None:
            return None
        return DescribeSendRequestsByUserIdResult()\
            .with_items(None if data.get('items') is None else [
                FriendRequest.from_dict(data.get('items')[i])
                for i in range(len(data.get('items')))
            ])\
            .with_next_page_token(data.get('nextPageToken'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "items": None if self.items is None else [
                self.items[i].to_dict() if self.items[i] else None
                for i in range(len(self.items))
            ],
            "nextPageToken": self.next_page_token,
        }


class GetSendRequestResult(core.Gs2Result):
    item: FriendRequest = None

    def with_item(self, item: FriendRequest) -> GetSendRequestResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[GetSendRequestResult]:
        if data is None:
            return None
        return GetSendRequestResult()\
            .with_item(FriendRequest.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class GetSendRequestByUserIdResult(core.Gs2Result):
    item: FriendRequest = None

    def with_item(self, item: FriendRequest) -> GetSendRequestByUserIdResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[GetSendRequestByUserIdResult]:
        if data is None:
            return None
        return GetSendRequestByUserIdResult()\
            .with_item(FriendRequest.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class SendRequestResult(core.Gs2Result):
    item: FriendRequest = None

    def with_item(self, item: FriendRequest) -> SendRequestResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[SendRequestResult]:
        if data is None:
            return None
        return SendRequestResult()\
            .with_item(FriendRequest.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class SendRequestByUserIdResult(core.Gs2Result):
    item: FriendRequest = None

    def with_item(self, item: FriendRequest) -> SendRequestByUserIdResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[SendRequestByUserIdResult]:
        if data is None:
            return None
        return SendRequestByUserIdResult()\
            .with_item(FriendRequest.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class DeleteRequestResult(core.Gs2Result):
    item: FriendRequest = None

    def with_item(self, item: FriendRequest) -> DeleteRequestResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[DeleteRequestResult]:
        if data is None:
            return None
        return DeleteRequestResult()\
            .with_item(FriendRequest.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class DeleteRequestByUserIdResult(core.Gs2Result):
    item: FriendRequest = None

    def with_item(self, item: FriendRequest) -> DeleteRequestByUserIdResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[DeleteRequestByUserIdResult]:
        if data is None:
            return None
        return DeleteRequestByUserIdResult()\
            .with_item(FriendRequest.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class DescribeReceiveRequestsResult(core.Gs2Result):
    items: List[FriendRequest] = None
    next_page_token: str = None

    def with_items(self, items: List[FriendRequest]) -> DescribeReceiveRequestsResult:
        self.items = items
        return self

    def with_next_page_token(self, next_page_token: str) -> DescribeReceiveRequestsResult:
        self.next_page_token = next_page_token
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[DescribeReceiveRequestsResult]:
        if data is None:
            return None
        return DescribeReceiveRequestsResult()\
            .with_items(None if data.get('items') is None else [
                FriendRequest.from_dict(data.get('items')[i])
                for i in range(len(data.get('items')))
            ])\
            .with_next_page_token(data.get('nextPageToken'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "items": None if self.items is None else [
                self.items[i].to_dict() if self.items[i] else None
                for i in range(len(self.items))
            ],
            "nextPageToken": self.next_page_token,
        }


class DescribeReceiveRequestsByUserIdResult(core.Gs2Result):
    items: List[FriendRequest] = None
    next_page_token: str = None

    def with_items(self, items: List[FriendRequest]) -> DescribeReceiveRequestsByUserIdResult:
        self.items = items
        return self

    def with_next_page_token(self, next_page_token: str) -> DescribeReceiveRequestsByUserIdResult:
        self.next_page_token = next_page_token
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[DescribeReceiveRequestsByUserIdResult]:
        if data is None:
            return None
        return DescribeReceiveRequestsByUserIdResult()\
            .with_items(None if data.get('items') is None else [
                FriendRequest.from_dict(data.get('items')[i])
                for i in range(len(data.get('items')))
            ])\
            .with_next_page_token(data.get('nextPageToken'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "items": None if self.items is None else [
                self.items[i].to_dict() if self.items[i] else None
                for i in range(len(self.items))
            ],
            "nextPageToken": self.next_page_token,
        }


class GetReceiveRequestResult(core.Gs2Result):
    item: FriendRequest = None

    def with_item(self, item: FriendRequest) -> GetReceiveRequestResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[GetReceiveRequestResult]:
        if data is None:
            return None
        return GetReceiveRequestResult()\
            .with_item(FriendRequest.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class GetReceiveRequestByUserIdResult(core.Gs2Result):
    item: FriendRequest = None

    def with_item(self, item: FriendRequest) -> GetReceiveRequestByUserIdResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[GetReceiveRequestByUserIdResult]:
        if data is None:
            return None
        return GetReceiveRequestByUserIdResult()\
            .with_item(FriendRequest.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class AcceptRequestResult(core.Gs2Result):
    item: FriendRequest = None

    def with_item(self, item: FriendRequest) -> AcceptRequestResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[AcceptRequestResult]:
        if data is None:
            return None
        return AcceptRequestResult()\
            .with_item(FriendRequest.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class AcceptRequestByUserIdResult(core.Gs2Result):
    item: FriendRequest = None

    def with_item(self, item: FriendRequest) -> AcceptRequestByUserIdResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[AcceptRequestByUserIdResult]:
        if data is None:
            return None
        return AcceptRequestByUserIdResult()\
            .with_item(FriendRequest.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class RejectRequestResult(core.Gs2Result):
    item: FriendRequest = None

    def with_item(self, item: FriendRequest) -> RejectRequestResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[RejectRequestResult]:
        if data is None:
            return None
        return RejectRequestResult()\
            .with_item(FriendRequest.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class RejectRequestByUserIdResult(core.Gs2Result):
    item: FriendRequest = None

    def with_item(self, item: FriendRequest) -> RejectRequestByUserIdResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[RejectRequestByUserIdResult]:
        if data is None:
            return None
        return RejectRequestByUserIdResult()\
            .with_item(FriendRequest.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class GetPublicProfileResult(core.Gs2Result):
    item: PublicProfile = None

    def with_item(self, item: PublicProfile) -> GetPublicProfileResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[GetPublicProfileResult]:
        if data is None:
            return None
        return GetPublicProfileResult()\
            .with_item(PublicProfile.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


# encoding: utf-8
#
# Copyright 2016 Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.

from gs2.core import *
from .request import *
from .result import *
import time


class Gs2ProjectWebSocketClient(web_socket.AbstractGs2WebSocketClient):

    def _create_account(
        self,
        request: CreateAccountRequest,
        callback: Callable[[AsyncResult[CreateAccountResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='account',
            function='createAccount',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.email is not None:
            body["email"] = request.email
        if request.full_name is not None:
            body["fullName"] = request.full_name
        if request.company_name is not None:
            body["companyName"] = request.company_name
        if request.password is not None:
            body["password"] = request.password
        if request.lang is not None:
            body["lang"] = request.lang

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=CreateAccountResult,
                callback=callback,
                body=body,
            )
        )

    def create_account(
        self,
        request: CreateAccountRequest,
    ) -> CreateAccountResult:
        async_result = []
        with timeout(30):
            self._create_account(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def create_account_async(
        self,
        request: CreateAccountRequest,
    ) -> CreateAccountResult:
        async_result = []
        self._create_account(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _verify(
        self,
        request: VerifyRequest,
        callback: Callable[[AsyncResult[VerifyResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='account',
            function='verify',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.verify_token is not None:
            body["verifyToken"] = request.verify_token

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=VerifyResult,
                callback=callback,
                body=body,
            )
        )

    def verify(
        self,
        request: VerifyRequest,
    ) -> VerifyResult:
        async_result = []
        with timeout(30):
            self._verify(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def verify_async(
        self,
        request: VerifyRequest,
    ) -> VerifyResult:
        async_result = []
        self._verify(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _sign_in(
        self,
        request: SignInRequest,
        callback: Callable[[AsyncResult[SignInResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='account',
            function='signIn',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.email is not None:
            body["email"] = request.email
        if request.password is not None:
            body["password"] = request.password
        if request.otp is not None:
            body["otp"] = request.otp

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=SignInResult,
                callback=callback,
                body=body,
            )
        )

    def sign_in(
        self,
        request: SignInRequest,
    ) -> SignInResult:
        async_result = []
        with timeout(30):
            self._sign_in(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def sign_in_async(
        self,
        request: SignInRequest,
    ) -> SignInResult:
        async_result = []
        self._sign_in(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _forget(
        self,
        request: ForgetRequest,
        callback: Callable[[AsyncResult[ForgetResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='account',
            function='forget',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.email is not None:
            body["email"] = request.email
        if request.lang is not None:
            body["lang"] = request.lang

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=ForgetResult,
                callback=callback,
                body=body,
            )
        )

    def forget(
        self,
        request: ForgetRequest,
    ) -> ForgetResult:
        async_result = []
        with timeout(30):
            self._forget(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def forget_async(
        self,
        request: ForgetRequest,
    ) -> ForgetResult:
        async_result = []
        self._forget(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _issue_password(
        self,
        request: IssuePasswordRequest,
        callback: Callable[[AsyncResult[IssuePasswordResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='account',
            function='issuePassword',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.issue_password_token is not None:
            body["issuePasswordToken"] = request.issue_password_token

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=IssuePasswordResult,
                callback=callback,
                body=body,
            )
        )

    def issue_password(
        self,
        request: IssuePasswordRequest,
    ) -> IssuePasswordResult:
        async_result = []
        with timeout(30):
            self._issue_password(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def issue_password_async(
        self,
        request: IssuePasswordRequest,
    ) -> IssuePasswordResult:
        async_result = []
        self._issue_password(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _update_account(
        self,
        request: UpdateAccountRequest,
        callback: Callable[[AsyncResult[UpdateAccountResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='account',
            function='updateAccount',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.email is not None:
            body["email"] = request.email
        if request.full_name is not None:
            body["fullName"] = request.full_name
        if request.company_name is not None:
            body["companyName"] = request.company_name
        if request.password is not None:
            body["password"] = request.password
        if request.account_token is not None:
            body["accountToken"] = request.account_token

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=UpdateAccountResult,
                callback=callback,
                body=body,
            )
        )

    def update_account(
        self,
        request: UpdateAccountRequest,
    ) -> UpdateAccountResult:
        async_result = []
        with timeout(30):
            self._update_account(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def update_account_async(
        self,
        request: UpdateAccountRequest,
    ) -> UpdateAccountResult:
        async_result = []
        self._update_account(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _enable_mfa(
        self,
        request: EnableMfaRequest,
        callback: Callable[[AsyncResult[EnableMfaResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='account',
            function='enableMfa',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.account_token is not None:
            body["accountToken"] = request.account_token

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=EnableMfaResult,
                callback=callback,
                body=body,
            )
        )

    def enable_mfa(
        self,
        request: EnableMfaRequest,
    ) -> EnableMfaResult:
        async_result = []
        with timeout(30):
            self._enable_mfa(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def enable_mfa_async(
        self,
        request: EnableMfaRequest,
    ) -> EnableMfaResult:
        async_result = []
        self._enable_mfa(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _challenge_mfa(
        self,
        request: ChallengeMfaRequest,
        callback: Callable[[AsyncResult[ChallengeMfaResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='account',
            function='challengeMfa',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.account_token is not None:
            body["accountToken"] = request.account_token
        if request.passcode is not None:
            body["passcode"] = request.passcode

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=ChallengeMfaResult,
                callback=callback,
                body=body,
            )
        )

    def challenge_mfa(
        self,
        request: ChallengeMfaRequest,
    ) -> ChallengeMfaResult:
        async_result = []
        with timeout(30):
            self._challenge_mfa(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def challenge_mfa_async(
        self,
        request: ChallengeMfaRequest,
    ) -> ChallengeMfaResult:
        async_result = []
        self._challenge_mfa(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _disable_mfa(
        self,
        request: DisableMfaRequest,
        callback: Callable[[AsyncResult[DisableMfaResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='account',
            function='disableMfa',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.account_token is not None:
            body["accountToken"] = request.account_token

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=DisableMfaResult,
                callback=callback,
                body=body,
            )
        )

    def disable_mfa(
        self,
        request: DisableMfaRequest,
    ) -> DisableMfaResult:
        async_result = []
        with timeout(30):
            self._disable_mfa(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def disable_mfa_async(
        self,
        request: DisableMfaRequest,
    ) -> DisableMfaResult:
        async_result = []
        self._disable_mfa(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _delete_account(
        self,
        request: DeleteAccountRequest,
        callback: Callable[[AsyncResult[DeleteAccountResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='account',
            function='deleteAccount',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.account_token is not None:
            body["accountToken"] = request.account_token

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=DeleteAccountResult,
                callback=callback,
                body=body,
            )
        )

    def delete_account(
        self,
        request: DeleteAccountRequest,
    ) -> DeleteAccountResult:
        async_result = []
        with timeout(30):
            self._delete_account(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def delete_account_async(
        self,
        request: DeleteAccountRequest,
    ) -> DeleteAccountResult:
        async_result = []
        self._delete_account(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _describe_projects(
        self,
        request: DescribeProjectsRequest,
        callback: Callable[[AsyncResult[DescribeProjectsResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='project',
            function='describeProjects',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.account_token is not None:
            body["accountToken"] = request.account_token
        if request.page_token is not None:
            body["pageToken"] = request.page_token
        if request.limit is not None:
            body["limit"] = request.limit

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=DescribeProjectsResult,
                callback=callback,
                body=body,
            )
        )

    def describe_projects(
        self,
        request: DescribeProjectsRequest,
    ) -> DescribeProjectsResult:
        async_result = []
        with timeout(30):
            self._describe_projects(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def describe_projects_async(
        self,
        request: DescribeProjectsRequest,
    ) -> DescribeProjectsResult:
        async_result = []
        self._describe_projects(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _create_project(
        self,
        request: CreateProjectRequest,
        callback: Callable[[AsyncResult[CreateProjectResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='project',
            function='createProject',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.account_token is not None:
            body["accountToken"] = request.account_token
        if request.name is not None:
            body["name"] = request.name
        if request.description is not None:
            body["description"] = request.description
        if request.plan is not None:
            body["plan"] = request.plan
        if request.currency is not None:
            body["currency"] = request.currency
        if request.activate_region_name is not None:
            body["activateRegionName"] = request.activate_region_name
        if request.billing_method_name is not None:
            body["billingMethodName"] = request.billing_method_name
        if request.enable_event_bridge is not None:
            body["enableEventBridge"] = request.enable_event_bridge
        if request.event_bridge_aws_account_id is not None:
            body["eventBridgeAwsAccountId"] = request.event_bridge_aws_account_id
        if request.event_bridge_aws_region is not None:
            body["eventBridgeAwsRegion"] = request.event_bridge_aws_region

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=CreateProjectResult,
                callback=callback,
                body=body,
            )
        )

    def create_project(
        self,
        request: CreateProjectRequest,
    ) -> CreateProjectResult:
        async_result = []
        with timeout(30):
            self._create_project(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def create_project_async(
        self,
        request: CreateProjectRequest,
    ) -> CreateProjectResult:
        async_result = []
        self._create_project(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _get_project(
        self,
        request: GetProjectRequest,
        callback: Callable[[AsyncResult[GetProjectResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='project',
            function='getProject',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.account_token is not None:
            body["accountToken"] = request.account_token
        if request.project_name is not None:
            body["projectName"] = request.project_name

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=GetProjectResult,
                callback=callback,
                body=body,
            )
        )

    def get_project(
        self,
        request: GetProjectRequest,
    ) -> GetProjectResult:
        async_result = []
        with timeout(30):
            self._get_project(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def get_project_async(
        self,
        request: GetProjectRequest,
    ) -> GetProjectResult:
        async_result = []
        self._get_project(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _get_project_token(
        self,
        request: GetProjectTokenRequest,
        callback: Callable[[AsyncResult[GetProjectTokenResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='project',
            function='getProjectToken',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.project_name is not None:
            body["projectName"] = request.project_name
        if request.account_token is not None:
            body["accountToken"] = request.account_token

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=GetProjectTokenResult,
                callback=callback,
                body=body,
            )
        )

    def get_project_token(
        self,
        request: GetProjectTokenRequest,
    ) -> GetProjectTokenResult:
        async_result = []
        with timeout(30):
            self._get_project_token(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def get_project_token_async(
        self,
        request: GetProjectTokenRequest,
    ) -> GetProjectTokenResult:
        async_result = []
        self._get_project_token(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _get_project_token_by_identifier(
        self,
        request: GetProjectTokenByIdentifierRequest,
        callback: Callable[[AsyncResult[GetProjectTokenByIdentifierResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='project',
            function='getProjectTokenByIdentifier',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.account_name is not None:
            body["accountName"] = request.account_name
        if request.project_name is not None:
            body["projectName"] = request.project_name
        if request.user_name is not None:
            body["userName"] = request.user_name
        if request.password is not None:
            body["password"] = request.password
        if request.otp is not None:
            body["otp"] = request.otp

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=GetProjectTokenByIdentifierResult,
                callback=callback,
                body=body,
            )
        )

    def get_project_token_by_identifier(
        self,
        request: GetProjectTokenByIdentifierRequest,
    ) -> GetProjectTokenByIdentifierResult:
        async_result = []
        with timeout(30):
            self._get_project_token_by_identifier(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def get_project_token_by_identifier_async(
        self,
        request: GetProjectTokenByIdentifierRequest,
    ) -> GetProjectTokenByIdentifierResult:
        async_result = []
        self._get_project_token_by_identifier(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _update_project(
        self,
        request: UpdateProjectRequest,
        callback: Callable[[AsyncResult[UpdateProjectResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='project',
            function='updateProject',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.account_token is not None:
            body["accountToken"] = request.account_token
        if request.project_name is not None:
            body["projectName"] = request.project_name
        if request.description is not None:
            body["description"] = request.description
        if request.plan is not None:
            body["plan"] = request.plan
        if request.billing_method_name is not None:
            body["billingMethodName"] = request.billing_method_name
        if request.enable_event_bridge is not None:
            body["enableEventBridge"] = request.enable_event_bridge
        if request.event_bridge_aws_account_id is not None:
            body["eventBridgeAwsAccountId"] = request.event_bridge_aws_account_id
        if request.event_bridge_aws_region is not None:
            body["eventBridgeAwsRegion"] = request.event_bridge_aws_region

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=UpdateProjectResult,
                callback=callback,
                body=body,
            )
        )

    def update_project(
        self,
        request: UpdateProjectRequest,
    ) -> UpdateProjectResult:
        async_result = []
        with timeout(30):
            self._update_project(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def update_project_async(
        self,
        request: UpdateProjectRequest,
    ) -> UpdateProjectResult:
        async_result = []
        self._update_project(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _activate_region(
        self,
        request: ActivateRegionRequest,
        callback: Callable[[AsyncResult[ActivateRegionResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='project',
            function='activateRegion',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.account_token is not None:
            body["accountToken"] = request.account_token
        if request.project_name is not None:
            body["projectName"] = request.project_name
        if request.region_name is not None:
            body["regionName"] = request.region_name

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=ActivateRegionResult,
                callback=callback,
                body=body,
            )
        )

    def activate_region(
        self,
        request: ActivateRegionRequest,
    ) -> ActivateRegionResult:
        async_result = []
        with timeout(30):
            self._activate_region(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def activate_region_async(
        self,
        request: ActivateRegionRequest,
    ) -> ActivateRegionResult:
        async_result = []
        self._activate_region(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _wait_activate_region(
        self,
        request: WaitActivateRegionRequest,
        callback: Callable[[AsyncResult[WaitActivateRegionResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='project',
            function='waitActivateRegion',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.project_name is not None:
            body["projectName"] = request.project_name
        if request.region_name is not None:
            body["regionName"] = request.region_name

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=WaitActivateRegionResult,
                callback=callback,
                body=body,
            )
        )

    def wait_activate_region(
        self,
        request: WaitActivateRegionRequest,
    ) -> WaitActivateRegionResult:
        async_result = []
        with timeout(30):
            self._wait_activate_region(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def wait_activate_region_async(
        self,
        request: WaitActivateRegionRequest,
    ) -> WaitActivateRegionResult:
        async_result = []
        self._wait_activate_region(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _delete_project(
        self,
        request: DeleteProjectRequest,
        callback: Callable[[AsyncResult[DeleteProjectResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='project',
            function='deleteProject',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.account_token is not None:
            body["accountToken"] = request.account_token
        if request.project_name is not None:
            body["projectName"] = request.project_name

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=DeleteProjectResult,
                callback=callback,
                body=body,
            )
        )

    def delete_project(
        self,
        request: DeleteProjectRequest,
    ) -> DeleteProjectResult:
        async_result = []
        with timeout(30):
            self._delete_project(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def delete_project_async(
        self,
        request: DeleteProjectRequest,
    ) -> DeleteProjectResult:
        async_result = []
        self._delete_project(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _describe_billing_methods(
        self,
        request: DescribeBillingMethodsRequest,
        callback: Callable[[AsyncResult[DescribeBillingMethodsResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='billingMethod',
            function='describeBillingMethods',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.account_token is not None:
            body["accountToken"] = request.account_token
        if request.page_token is not None:
            body["pageToken"] = request.page_token
        if request.limit is not None:
            body["limit"] = request.limit

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=DescribeBillingMethodsResult,
                callback=callback,
                body=body,
            )
        )

    def describe_billing_methods(
        self,
        request: DescribeBillingMethodsRequest,
    ) -> DescribeBillingMethodsResult:
        async_result = []
        with timeout(30):
            self._describe_billing_methods(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def describe_billing_methods_async(
        self,
        request: DescribeBillingMethodsRequest,
    ) -> DescribeBillingMethodsResult:
        async_result = []
        self._describe_billing_methods(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _create_billing_method(
        self,
        request: CreateBillingMethodRequest,
        callback: Callable[[AsyncResult[CreateBillingMethodResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='billingMethod',
            function='createBillingMethod',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.account_token is not None:
            body["accountToken"] = request.account_token
        if request.description is not None:
            body["description"] = request.description
        if request.method_type is not None:
            body["methodType"] = request.method_type
        if request.card_customer_id is not None:
            body["cardCustomerId"] = request.card_customer_id
        if request.partner_id is not None:
            body["partnerId"] = request.partner_id

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=CreateBillingMethodResult,
                callback=callback,
                body=body,
            )
        )

    def create_billing_method(
        self,
        request: CreateBillingMethodRequest,
    ) -> CreateBillingMethodResult:
        async_result = []
        with timeout(30):
            self._create_billing_method(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def create_billing_method_async(
        self,
        request: CreateBillingMethodRequest,
    ) -> CreateBillingMethodResult:
        async_result = []
        self._create_billing_method(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _get_billing_method(
        self,
        request: GetBillingMethodRequest,
        callback: Callable[[AsyncResult[GetBillingMethodResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='billingMethod',
            function='getBillingMethod',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.account_token is not None:
            body["accountToken"] = request.account_token
        if request.billing_method_name is not None:
            body["billingMethodName"] = request.billing_method_name

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=GetBillingMethodResult,
                callback=callback,
                body=body,
            )
        )

    def get_billing_method(
        self,
        request: GetBillingMethodRequest,
    ) -> GetBillingMethodResult:
        async_result = []
        with timeout(30):
            self._get_billing_method(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def get_billing_method_async(
        self,
        request: GetBillingMethodRequest,
    ) -> GetBillingMethodResult:
        async_result = []
        self._get_billing_method(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _update_billing_method(
        self,
        request: UpdateBillingMethodRequest,
        callback: Callable[[AsyncResult[UpdateBillingMethodResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='billingMethod',
            function='updateBillingMethod',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.account_token is not None:
            body["accountToken"] = request.account_token
        if request.billing_method_name is not None:
            body["billingMethodName"] = request.billing_method_name
        if request.description is not None:
            body["description"] = request.description

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=UpdateBillingMethodResult,
                callback=callback,
                body=body,
            )
        )

    def update_billing_method(
        self,
        request: UpdateBillingMethodRequest,
    ) -> UpdateBillingMethodResult:
        async_result = []
        with timeout(30):
            self._update_billing_method(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def update_billing_method_async(
        self,
        request: UpdateBillingMethodRequest,
    ) -> UpdateBillingMethodResult:
        async_result = []
        self._update_billing_method(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _delete_billing_method(
        self,
        request: DeleteBillingMethodRequest,
        callback: Callable[[AsyncResult[DeleteBillingMethodResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='billingMethod',
            function='deleteBillingMethod',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.account_token is not None:
            body["accountToken"] = request.account_token
        if request.billing_method_name is not None:
            body["billingMethodName"] = request.billing_method_name

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=DeleteBillingMethodResult,
                callback=callback,
                body=body,
            )
        )

    def delete_billing_method(
        self,
        request: DeleteBillingMethodRequest,
    ) -> DeleteBillingMethodResult:
        async_result = []
        with timeout(30):
            self._delete_billing_method(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def delete_billing_method_async(
        self,
        request: DeleteBillingMethodRequest,
    ) -> DeleteBillingMethodResult:
        async_result = []
        self._delete_billing_method(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _describe_receipts(
        self,
        request: DescribeReceiptsRequest,
        callback: Callable[[AsyncResult[DescribeReceiptsResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='receipt',
            function='describeReceipts',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.account_token is not None:
            body["accountToken"] = request.account_token
        if request.page_token is not None:
            body["pageToken"] = request.page_token
        if request.limit is not None:
            body["limit"] = request.limit

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=DescribeReceiptsResult,
                callback=callback,
                body=body,
            )
        )

    def describe_receipts(
        self,
        request: DescribeReceiptsRequest,
    ) -> DescribeReceiptsResult:
        async_result = []
        with timeout(30):
            self._describe_receipts(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def describe_receipts_async(
        self,
        request: DescribeReceiptsRequest,
    ) -> DescribeReceiptsResult:
        async_result = []
        self._describe_receipts(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _describe_billings(
        self,
        request: DescribeBillingsRequest,
        callback: Callable[[AsyncResult[DescribeBillingsResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='billing',
            function='describeBillings',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.account_token is not None:
            body["accountToken"] = request.account_token
        if request.project_name is not None:
            body["projectName"] = request.project_name
        if request.year is not None:
            body["year"] = request.year
        if request.month is not None:
            body["month"] = request.month
        if request.region is not None:
            body["region"] = request.region
        if request.service is not None:
            body["service"] = request.service

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=DescribeBillingsResult,
                callback=callback,
                body=body,
            )
        )

    def describe_billings(
        self,
        request: DescribeBillingsRequest,
    ) -> DescribeBillingsResult:
        async_result = []
        with timeout(30):
            self._describe_billings(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def describe_billings_async(
        self,
        request: DescribeBillingsRequest,
    ) -> DescribeBillingsResult:
        async_result = []
        self._describe_billings(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _describe_dump_progresses(
        self,
        request: DescribeDumpProgressesRequest,
        callback: Callable[[AsyncResult[DescribeDumpProgressesResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='dumpProgress',
            function='describeDumpProgresses',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.page_token is not None:
            body["pageToken"] = request.page_token
        if request.limit is not None:
            body["limit"] = request.limit

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=DescribeDumpProgressesResult,
                callback=callback,
                body=body,
            )
        )

    def describe_dump_progresses(
        self,
        request: DescribeDumpProgressesRequest,
    ) -> DescribeDumpProgressesResult:
        async_result = []
        with timeout(30):
            self._describe_dump_progresses(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def describe_dump_progresses_async(
        self,
        request: DescribeDumpProgressesRequest,
    ) -> DescribeDumpProgressesResult:
        async_result = []
        self._describe_dump_progresses(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _get_dump_progress(
        self,
        request: GetDumpProgressRequest,
        callback: Callable[[AsyncResult[GetDumpProgressResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='dumpProgress',
            function='getDumpProgress',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.transaction_id is not None:
            body["transactionId"] = request.transaction_id

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=GetDumpProgressResult,
                callback=callback,
                body=body,
            )
        )

    def get_dump_progress(
        self,
        request: GetDumpProgressRequest,
    ) -> GetDumpProgressResult:
        async_result = []
        with timeout(30):
            self._get_dump_progress(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def get_dump_progress_async(
        self,
        request: GetDumpProgressRequest,
    ) -> GetDumpProgressResult:
        async_result = []
        self._get_dump_progress(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _wait_dump_user_data(
        self,
        request: WaitDumpUserDataRequest,
        callback: Callable[[AsyncResult[WaitDumpUserDataResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='dumpProgress',
            function='waitDumpUserData',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.owner_id is not None:
            body["ownerId"] = request.owner_id
        if request.transaction_id is not None:
            body["transactionId"] = request.transaction_id
        if request.user_id is not None:
            body["userId"] = request.user_id
        if request.microservice_name is not None:
            body["microserviceName"] = request.microservice_name
        if request.time_offset_token is not None:
            body["timeOffsetToken"] = request.time_offset_token

        if request.request_id:
            body["xGs2RequestId"] = request.request_id
        if request.duplication_avoider:
            body["xGs2DuplicationAvoider"] = request.duplication_avoider

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=WaitDumpUserDataResult,
                callback=callback,
                body=body,
            )
        )

    def wait_dump_user_data(
        self,
        request: WaitDumpUserDataRequest,
    ) -> WaitDumpUserDataResult:
        async_result = []
        with timeout(30):
            self._wait_dump_user_data(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def wait_dump_user_data_async(
        self,
        request: WaitDumpUserDataRequest,
    ) -> WaitDumpUserDataResult:
        async_result = []
        self._wait_dump_user_data(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _archive_dump_user_data(
        self,
        request: ArchiveDumpUserDataRequest,
        callback: Callable[[AsyncResult[ArchiveDumpUserDataResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='dumpProgress',
            function='archiveDumpUserData',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.owner_id is not None:
            body["ownerId"] = request.owner_id
        if request.transaction_id is not None:
            body["transactionId"] = request.transaction_id

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=ArchiveDumpUserDataResult,
                callback=callback,
                body=body,
            )
        )

    def archive_dump_user_data(
        self,
        request: ArchiveDumpUserDataRequest,
    ) -> ArchiveDumpUserDataResult:
        async_result = []
        with timeout(30):
            self._archive_dump_user_data(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def archive_dump_user_data_async(
        self,
        request: ArchiveDumpUserDataRequest,
    ) -> ArchiveDumpUserDataResult:
        async_result = []
        self._archive_dump_user_data(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _dump_user_data(
        self,
        request: DumpUserDataRequest,
        callback: Callable[[AsyncResult[DumpUserDataResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='dumpProgress',
            function='dumpUserData',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.user_id is not None:
            body["userId"] = request.user_id
        if request.time_offset_token is not None:
            body["timeOffsetToken"] = request.time_offset_token

        if request.request_id:
            body["xGs2RequestId"] = request.request_id
        if request.duplication_avoider:
            body["xGs2DuplicationAvoider"] = request.duplication_avoider

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=DumpUserDataResult,
                callback=callback,
                body=body,
            )
        )

    def dump_user_data(
        self,
        request: DumpUserDataRequest,
    ) -> DumpUserDataResult:
        async_result = []
        with timeout(30):
            self._dump_user_data(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def dump_user_data_async(
        self,
        request: DumpUserDataRequest,
    ) -> DumpUserDataResult:
        async_result = []
        self._dump_user_data(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _get_dump_user_data(
        self,
        request: GetDumpUserDataRequest,
        callback: Callable[[AsyncResult[GetDumpUserDataResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='dumpProgress',
            function='getDumpUserData',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.transaction_id is not None:
            body["transactionId"] = request.transaction_id

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=GetDumpUserDataResult,
                callback=callback,
                body=body,
            )
        )

    def get_dump_user_data(
        self,
        request: GetDumpUserDataRequest,
    ) -> GetDumpUserDataResult:
        async_result = []
        with timeout(30):
            self._get_dump_user_data(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def get_dump_user_data_async(
        self,
        request: GetDumpUserDataRequest,
    ) -> GetDumpUserDataResult:
        async_result = []
        self._get_dump_user_data(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _describe_clean_progresses(
        self,
        request: DescribeCleanProgressesRequest,
        callback: Callable[[AsyncResult[DescribeCleanProgressesResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='cleanProgress',
            function='describeCleanProgresses',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.page_token is not None:
            body["pageToken"] = request.page_token
        if request.limit is not None:
            body["limit"] = request.limit

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=DescribeCleanProgressesResult,
                callback=callback,
                body=body,
            )
        )

    def describe_clean_progresses(
        self,
        request: DescribeCleanProgressesRequest,
    ) -> DescribeCleanProgressesResult:
        async_result = []
        with timeout(30):
            self._describe_clean_progresses(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def describe_clean_progresses_async(
        self,
        request: DescribeCleanProgressesRequest,
    ) -> DescribeCleanProgressesResult:
        async_result = []
        self._describe_clean_progresses(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _get_clean_progress(
        self,
        request: GetCleanProgressRequest,
        callback: Callable[[AsyncResult[GetCleanProgressResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='cleanProgress',
            function='getCleanProgress',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.transaction_id is not None:
            body["transactionId"] = request.transaction_id

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=GetCleanProgressResult,
                callback=callback,
                body=body,
            )
        )

    def get_clean_progress(
        self,
        request: GetCleanProgressRequest,
    ) -> GetCleanProgressResult:
        async_result = []
        with timeout(30):
            self._get_clean_progress(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def get_clean_progress_async(
        self,
        request: GetCleanProgressRequest,
    ) -> GetCleanProgressResult:
        async_result = []
        self._get_clean_progress(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _wait_clean_user_data(
        self,
        request: WaitCleanUserDataRequest,
        callback: Callable[[AsyncResult[WaitCleanUserDataResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='cleanProgress',
            function='waitCleanUserData',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.transaction_id is not None:
            body["transactionId"] = request.transaction_id
        if request.user_id is not None:
            body["userId"] = request.user_id
        if request.microservice_name is not None:
            body["microserviceName"] = request.microservice_name
        if request.time_offset_token is not None:
            body["timeOffsetToken"] = request.time_offset_token

        if request.request_id:
            body["xGs2RequestId"] = request.request_id
        if request.duplication_avoider:
            body["xGs2DuplicationAvoider"] = request.duplication_avoider

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=WaitCleanUserDataResult,
                callback=callback,
                body=body,
            )
        )

    def wait_clean_user_data(
        self,
        request: WaitCleanUserDataRequest,
    ) -> WaitCleanUserDataResult:
        async_result = []
        with timeout(30):
            self._wait_clean_user_data(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def wait_clean_user_data_async(
        self,
        request: WaitCleanUserDataRequest,
    ) -> WaitCleanUserDataResult:
        async_result = []
        self._wait_clean_user_data(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _clean_user_data(
        self,
        request: CleanUserDataRequest,
        callback: Callable[[AsyncResult[CleanUserDataResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='cleanProgress',
            function='cleanUserData',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.user_id is not None:
            body["userId"] = request.user_id
        if request.time_offset_token is not None:
            body["timeOffsetToken"] = request.time_offset_token

        if request.request_id:
            body["xGs2RequestId"] = request.request_id
        if request.duplication_avoider:
            body["xGs2DuplicationAvoider"] = request.duplication_avoider

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=CleanUserDataResult,
                callback=callback,
                body=body,
            )
        )

    def clean_user_data(
        self,
        request: CleanUserDataRequest,
    ) -> CleanUserDataResult:
        async_result = []
        with timeout(30):
            self._clean_user_data(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def clean_user_data_async(
        self,
        request: CleanUserDataRequest,
    ) -> CleanUserDataResult:
        async_result = []
        self._clean_user_data(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _describe_import_progresses(
        self,
        request: DescribeImportProgressesRequest,
        callback: Callable[[AsyncResult[DescribeImportProgressesResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='importProgress',
            function='describeImportProgresses',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.page_token is not None:
            body["pageToken"] = request.page_token
        if request.limit is not None:
            body["limit"] = request.limit

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=DescribeImportProgressesResult,
                callback=callback,
                body=body,
            )
        )

    def describe_import_progresses(
        self,
        request: DescribeImportProgressesRequest,
    ) -> DescribeImportProgressesResult:
        async_result = []
        with timeout(30):
            self._describe_import_progresses(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def describe_import_progresses_async(
        self,
        request: DescribeImportProgressesRequest,
    ) -> DescribeImportProgressesResult:
        async_result = []
        self._describe_import_progresses(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _get_import_progress(
        self,
        request: GetImportProgressRequest,
        callback: Callable[[AsyncResult[GetImportProgressResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='importProgress',
            function='getImportProgress',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.transaction_id is not None:
            body["transactionId"] = request.transaction_id

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=GetImportProgressResult,
                callback=callback,
                body=body,
            )
        )

    def get_import_progress(
        self,
        request: GetImportProgressRequest,
    ) -> GetImportProgressResult:
        async_result = []
        with timeout(30):
            self._get_import_progress(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def get_import_progress_async(
        self,
        request: GetImportProgressRequest,
    ) -> GetImportProgressResult:
        async_result = []
        self._get_import_progress(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _wait_import_user_data(
        self,
        request: WaitImportUserDataRequest,
        callback: Callable[[AsyncResult[WaitImportUserDataResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='importProgress',
            function='waitImportUserData',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.transaction_id is not None:
            body["transactionId"] = request.transaction_id
        if request.user_id is not None:
            body["userId"] = request.user_id
        if request.microservice_name is not None:
            body["microserviceName"] = request.microservice_name
        if request.time_offset_token is not None:
            body["timeOffsetToken"] = request.time_offset_token

        if request.request_id:
            body["xGs2RequestId"] = request.request_id
        if request.duplication_avoider:
            body["xGs2DuplicationAvoider"] = request.duplication_avoider

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=WaitImportUserDataResult,
                callback=callback,
                body=body,
            )
        )

    def wait_import_user_data(
        self,
        request: WaitImportUserDataRequest,
    ) -> WaitImportUserDataResult:
        async_result = []
        with timeout(30):
            self._wait_import_user_data(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def wait_import_user_data_async(
        self,
        request: WaitImportUserDataRequest,
    ) -> WaitImportUserDataResult:
        async_result = []
        self._wait_import_user_data(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _prepare_import_user_data(
        self,
        request: PrepareImportUserDataRequest,
        callback: Callable[[AsyncResult[PrepareImportUserDataResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='importProgress',
            function='prepareImportUserData',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.user_id is not None:
            body["userId"] = request.user_id
        if request.time_offset_token is not None:
            body["timeOffsetToken"] = request.time_offset_token

        if request.request_id:
            body["xGs2RequestId"] = request.request_id
        if request.duplication_avoider:
            body["xGs2DuplicationAvoider"] = request.duplication_avoider

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=PrepareImportUserDataResult,
                callback=callback,
                body=body,
            )
        )

    def prepare_import_user_data(
        self,
        request: PrepareImportUserDataRequest,
    ) -> PrepareImportUserDataResult:
        async_result = []
        with timeout(30):
            self._prepare_import_user_data(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def prepare_import_user_data_async(
        self,
        request: PrepareImportUserDataRequest,
    ) -> PrepareImportUserDataResult:
        async_result = []
        self._prepare_import_user_data(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _import_user_data(
        self,
        request: ImportUserDataRequest,
        callback: Callable[[AsyncResult[ImportUserDataResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='importProgress',
            function='importUserData',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.user_id is not None:
            body["userId"] = request.user_id
        if request.upload_token is not None:
            body["uploadToken"] = request.upload_token
        if request.time_offset_token is not None:
            body["timeOffsetToken"] = request.time_offset_token

        if request.request_id:
            body["xGs2RequestId"] = request.request_id
        if request.duplication_avoider:
            body["xGs2DuplicationAvoider"] = request.duplication_avoider

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=ImportUserDataResult,
                callback=callback,
                body=body,
            )
        )

    def import_user_data(
        self,
        request: ImportUserDataRequest,
    ) -> ImportUserDataResult:
        async_result = []
        with timeout(30):
            self._import_user_data(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def import_user_data_async(
        self,
        request: ImportUserDataRequest,
    ) -> ImportUserDataResult:
        async_result = []
        self._import_user_data(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _describe_import_error_logs(
        self,
        request: DescribeImportErrorLogsRequest,
        callback: Callable[[AsyncResult[DescribeImportErrorLogsResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='importErrorLog',
            function='describeImportErrorLogs',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.transaction_id is not None:
            body["transactionId"] = request.transaction_id
        if request.page_token is not None:
            body["pageToken"] = request.page_token
        if request.limit is not None:
            body["limit"] = request.limit

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=DescribeImportErrorLogsResult,
                callback=callback,
                body=body,
            )
        )

    def describe_import_error_logs(
        self,
        request: DescribeImportErrorLogsRequest,
    ) -> DescribeImportErrorLogsResult:
        async_result = []
        with timeout(30):
            self._describe_import_error_logs(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def describe_import_error_logs_async(
        self,
        request: DescribeImportErrorLogsRequest,
    ) -> DescribeImportErrorLogsResult:
        async_result = []
        self._describe_import_error_logs(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _get_import_error_log(
        self,
        request: GetImportErrorLogRequest,
        callback: Callable[[AsyncResult[GetImportErrorLogResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="project",
            component='importErrorLog',
            function='getImportErrorLog',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.transaction_id is not None:
            body["transactionId"] = request.transaction_id
        if request.error_log_name is not None:
            body["errorLogName"] = request.error_log_name

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            web_socket.NetworkJob(
                request_id=request_id,
                result_type=GetImportErrorLogResult,
                callback=callback,
                body=body,
            )
        )

    def get_import_error_log(
        self,
        request: GetImportErrorLogRequest,
    ) -> GetImportErrorLogResult:
        async_result = []
        with timeout(30):
            self._get_import_error_log(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def get_import_error_log_async(
        self,
        request: GetImportErrorLogRequest,
    ) -> GetImportErrorLogResult:
        async_result = []
        self._get_import_error_log(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


# encoding: utf-8
#
# Copyright 2016 Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
import time
from collections import deque
from typing import Dict, Any, Callable, TypeVar, Type, Deque, Generic
from netrc import netrc, NetrcParseError  # メインスレッド以外でimportするとデッドロックするらしい https://github.com/kennethreitz/requests/issues/2925

import requests

from ..core.exception import *
from ..core.model import *
from ..core.util import timeout

T = TypeVar('T')


def _parse_response(response: requests.Response) -> Dict[str, Any]:
    if response.status_code == 200:
        try:
            import json
            return json.loads(response.text)
        except ValueError:
            from ..core.exception import UnknownException
            raise UnknownException(response.text)
    elif response.status_code == 400:
        from ..core.exception import BadRequestException
        raise BadRequestException(response.text)
    elif response.status_code == 401:
        from ..core.exception import UnauthorizedException
        raise UnauthorizedException(response.text)
    elif response.status_code == 402:
        from ..core.exception import QuotaExceedException
        raise QuotaExceedException(response.text)
    elif response.status_code == 404:
        from ..core.exception import NotFoundException
        raise NotFoundException(response.text)
    elif response.status_code == 409:
        from ..core.exception import ConflictException
        raise ConflictException(response.text)
    elif response.status_code == 500:
        from ..core.exception import InternalServerErrorException
        raise InternalServerErrorException(response.text)
    elif response.status_code == 502:
        from ..core.exception import BadGatewayException
        raise BadGatewayException(response.text)
    elif response.status_code == 503:
        from ..core.exception import ServiceUnavailableException
        raise ServiceUnavailableException('')
    elif response.status_code == 504:
        from ..core.exception import RequestTimeoutException
        raise RequestTimeoutException(response.text)
    else:
        from ..core.exception import UnknownException
        raise UnknownException(response.text)


class NetworkJob:

    def __init__(
            self,
            url: str,
            method: str,
            result_type: Type[T],
            callback: Callable[[AsyncResult[T]], None],
            headers: Dict[str, Any] = None,
            query_strings: Dict[str, Any] = None,
            body: Dict[str, Any] = None,
    ):
        if headers is None:
            headers = {}
        if query_strings is None:
            query_strings = {}
        if body is None:
            body = {}

        self._url = url
        self._method = method
        self._result_type = result_type
        self._callback = callback
        self._headers = headers
        self._query_strings = query_strings
        self._body = body

    @property
    def url(self) -> str:
        return self._url

    @property
    def method(self) -> str:
        return self._method

    @property
    def result_type(self) -> Type[T]:
        return self._result_type

    @property
    def callback(self) -> Callable[[T], None]:
        return self._callback

    @property
    def headers(self) -> Dict[str, Any]:
        return self._headers

    @property
    def query_strings(self) -> Dict[str, Any]:
        return self._query_strings

    @property
    def body(self) -> Dict[str, Any]:
        return self._body


class Gs2RestSession(ISession):

    def __init__(
            self,
            credential: IGs2Credential,
            region: str,
    ):
        """
        コンストラクタ
        :param credential: クレデンシャル
        :param region: クレデンシャル
        """
        super().__init__()
        self._credential = credential
        self._project_token = None
        self._region = region
        self._connection = None
        self._connection_thread = None
        self._job_queue = deque()

    @property
    def credential(self) -> IGs2Credential:
        return self._credential

    @property
    def project_token(self) -> str:
        return self._project_token

    @property
    def region(self) -> str:
        return self._region

    @property
    def connection(self) -> requests.Session:
        return self._connection

    def _send(self, job: NetworkJob):
        if not self._connection:
            raise BrokenPipeError()

        if job.method == 'GET':
            response = self.connection.get(
                url=job.url,
                headers=job.headers,
                params=job.query_strings,
            )
        elif job.method == 'POST':
            response = self.connection.post(
                url=job.url,
                headers=job.headers,
                json=job.body,
            )
        elif job.method == 'PUT':
            response = self.connection.put(
                url=job.url,
                headers=job.headers,
                json=job.body,
            )
        elif job.method == 'DELETE':
            response = self.connection.delete(
                url=job.url,
                headers=job.headers,
                params=job.query_strings,
            )
        else:
            raise AttributeError()

        try:
            job.callback(
                AsyncResult(
                    result=job.result_type.from_dict(_parse_response(response)),
                )
            )
        except Gs2Exception as e:
            job.callback(
                AsyncResult(
                    error=e,
                )
            )

    def send(
            self,
            job: NetworkJob,
            is_blocking: bool = False
    ):
        if not self._connection:
            raise BrokenPipeError()
        if is_blocking:
            self._send(job)
            pass
        else:
            self._job_queue.append(job)

    def _connect(
            self,
            callback: Callable[[AsyncResult[LoginResult]], None],
            is_blocking: bool = False,
    ):
        try:
            if not self._connection:
                import threading
                self._connection = requests.Session()
                self._connection_thread = threading.Thread(target=receive_handler, args=(self, self._job_queue))
                self._connection_thread.start()

                if isinstance(self.credential, ProjectTokenGs2Credential):
                    self._project_token = self.credential.project_token
                    callback(
                        AsyncResult(
                        )
                    )
                else:
                    url = Gs2Constant.ENDPOINT_HOST.format(
                        service='identifier',
                        region=self.region,
                    ) + "/projectToken/login"
                    body = {
                        "client_id": self.credential.client_id,
                        "client_secret": self.credential.client_secret,
                    }

                    _job = NetworkJob(
                        url=url,
                        method='POST',
                        result_type=LoginResult,
                        callback=callback,
                        body=body,
                    )

                    self.send(
                        job=_job,
                        is_blocking=is_blocking,
                    )
            else:
                callback(
                    AsyncResult(
                        error=BrokenPipeError()
                    )
                )
        except Exception as e:
            callback(
                AsyncResult(
                    error=e
                )
            )

    def connect(self):
        if self._connection:
            return

        async_result = []
        with timeout(30):
            self._connect(
                lambda result: async_result.append(result),
                is_blocking=True,
            )

        if async_result[0].error:
            self.disconnect()
            raise async_result[0].error

        if self._project_token is None:
            self._project_token = async_result[0].result.access_token

    async def connect_async(self):
        if self._connection:
            return

        async_result = []
        self._connect(
            lambda result: async_result.append(result),
            is_blocking=False,
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error

        if self._project_token is None:
            self._project_token = async_result[0].result.access_token

    def disconnect(self):
        if self._connection:
            self._connection.close()
            self._connection = None
        for job in self._job_queue:
            job.callback(
                AsyncResult(
                    error=BrokenPipeError(),
                )
            )
        self._job_queue.clear()
        self._project_token = None


class AbstractGs2RestClient(object):

    def __init__(self, session):
        """
        コンストラクタ
        :param session: 認証情報
        :type session: gs2_core_client.model.RestSession.RestSession
        """
        self._session = session

    @property
    def session(self) -> Gs2RestSession:
        return self._session

    def _create_authorized_headers(self) -> Dict[str, Any]:
        return {
            'X-GS2-CLIENT-ID': self._session.credential.client_id,
            'Authorization': 'Bearer {}'.format(self._session.project_token),
        }


def receive_handler(
        session: Gs2RestSession,
        job_queue: Deque[NetworkJob],
):
    while session.connection:
        try:
            job = job_queue.popleft()
            session._send(job)
        except IndexError:
            time.sleep(0.01)

